/*
 *  Copyright 2013 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.skinfactory');

/**
 * Do not call this method. A unique instance of org.ametys.skinfactory.SkinParametersManager is used. Use org.ametys.skinfactory.SkinParametersManager.getInstance() instead. 
 * @constructor
 * @class This class has a single instance. Use org.ametys.skinfactory.SkinParametersManager.getInstance to get it. 
 */
org.ametys.skinfactory.SkinParametersManager = function ()
{
	this._currentParametersValues = {};
}

org.ametys.skinfactory.SkinParametersManager._instance;

/**
 * This method returns the unique instance of the org.ametys.skinfactory.SkinParametersManager
 * @static
 * @returns {org.ametys.skinfactory.SkinParametersManager} The unique instance of the org.ametys.skinfactory.SkinParametersManager class.
 */
org.ametys.skinfactory.SkinParametersManager.getInstance = function()
{
	if (org.ametys.skinfactory.SkinParametersManager._instance == null)
	{
		org.ametys.skinfactory.SkinParametersManager._instance = new org.ametys.skinfactory.SkinParametersManager();		
	}
	return org.ametys.skinfactory.SkinParametersManager._instance;
}

/**
 * @private
 * @property _currentParametersValues The current values of skin parameters. Can not be null.
 */
org.ametys.skinfactory.SkinParametersManager.prototype._currentParametersValues;

/**
 * Get the current values of skin parameters.
 * @return The current values in a Map
 */
org.ametys.skinfactory.SkinParametersManager.prototype.getParametersValues = function()
{
	return this._currentParametersValues;
}

/**
 * Get the current value of skin parameter. Can be null.
 * @param id The parameter's id
 * @return The current value or null.
 */
org.ametys.skinfactory.SkinParametersManager.prototype.getParameterValue = function (id)
{
	if (this._currentParametersValues[id])
	{
		return this._currentParametersValues[id];
	}
	
	return null;
}

org.ametys.skinfactory.SkinParametersManager.prototype.loadParametersValues = function(siteName, callback, scope)
{
    org.ametys.servercomm.ServerComm.getInstance().suspend();
    
    var serverMessage1 = new org.ametys.servercomm.ServerMessage("skinfactory", 'skin/parameters-values', {'siteName': siteName}, org.ametys.servercomm.ServerComm.PRIORITY_MAJOR, this._getParameterValueCb, this, [siteName, callback, scope], "text");
    org.ametys.servercomm.ServerComm.getInstance().send(serverMessage1);
    
    var serverMessage2 = new org.ametys.servercomm.ServerMessage("skinfactory", 'skin/colors', {'siteName': siteName}, org.ametys.servercomm.ServerComm.PRIORITY_MAJOR, this._getColorsCb, this, [], "text");
    org.ametys.servercomm.ServerComm.getInstance().send(serverMessage2);
    
    org.ametys.servercomm.ServerComm.getInstance().restart();
}

org.ametys.skinfactory.SkinParametersManager.prototype._getParameterValueCb = function (response, args)
{
    if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key='PLUGINS_SKINFACTORY_GET_PARAMETER_ERROR'/>", response, "org.ametys.skinfactory.SkinParametersManager"))
    {
        return;
    }

    var values = Ext.util.JSON.decode(response[org.ametys.servercomm.ServerComm.xmlTextContent]);
    this.updateParametersValues (values);
    
    var targets = org.ametys.messagebus.bus.MessageBuilder.getInstance().createTargets('skin-factory', {'id': args[0]});
    var event = org.ametys.messagebus.bus.MessageBuilder.getInstance().createMessage("skinParametersInitialized", {}, targets);
    org.ametys.messagebus.MessageBus.getInstance().fireMessages([event]);
    
    var callback = args[1];
    var scope = args[2];
    
    if (typeof (callback) == 'function')
    {
        callback.call(scope);
    }
}

org.ametys.skinfactory.SkinParametersManager.prototype._getColorsCb = function (response, args)
{
    if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key='PLUGINS_SKINFACTORY_GET_COLOR_THEME_ERROR'/>", response, "org.ametys.skinfactory.SkinParametersManager"))
    {
        return;
    }

    var result = Ext.util.JSON.decode(response[org.ametys.servercomm.ServerComm.xmlTextContent]);
    if (result.themeId)
    {
        this.updateColorTheme (result.themeId, result.themeColors);
    }
}
    

org.ametys.skinfactory.SkinParametersManager.prototype.updateParametersValues = function(values)
{
	this._currentParametersValues = values;
}

/**
 * @private
 * @property _currentColorTheme The current color theme. Can be null
 */
org.ametys.skinfactory.SkinParametersManager.prototype._currentColorTheme;
org.ametys.skinfactory.SkinParametersManager.prototype._themeColors = [];

org.ametys.skinfactory.SkinParametersManager.prototype.updateColorTheme = function (themeId, themeColors)
{
	this._currentColorTheme = themeId;
	this._themeColors = themeColors;
}

/**
 * Get the current color theme
 */
org.ametys.skinfactory.SkinParametersManager.prototype.getCurrentColorTheme = function ()
{
	return this._currentColorTheme;
}

/**
 * Get the current color theme
 */
org.ametys.skinfactory.SkinParametersManager.prototype.getThemeColors = function ()
{
	return this._themeColors;
}

// ------------------------------------------------------------------------------
org.ametys.skinfactory.SkinParametersManager.prototype._defaultColors;
org.ametys.skinfactory.SkinParametersManager.prototype.getDefaultColors = function ()
{
	if (this._defaultColors == null)
	{
		var serverMessage = new org.ametys.servercomm.ServerMessage("skinfactory", 'skin/colors', {'siteName': context.parameters.siteName}, org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS, null, this, null, "text");
		var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);

		// TODO Handle errors
		
		var result = Ext.util.JSON.decode(response[org.ametys.servercomm.ServerComm.xmlTextContent]);
		if (result.colors)
		{
			this._defaultColors = result.colors;
		}
	}
	
	return this._defaultColors;
}

//------------------------------------------------------------------------------
org.ametys.skinfactory.SkinParametersManager.prototype._languages;

/**
 * Get the available languages.
 */
org.ametys.skinfactory.SkinParametersManager.prototype.getLanguages = function ()
{
	if (this._languages == null)
	{
		var serverMessage= new org.ametys.servercomm.ServerMessage("skinfactory", 'skin/languages', {siteName: context.parameters.siteName}, org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS, null, this, null, "text");
		var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
		
		if (response != null)
		{
			this._languages = Ext.util.JSON.decode(response[org.ametys.servercomm.ServerComm.xmlTextContent]);
		}
	}
	
	return this._languages;
}

/**
 * Get the language of given code
 * @param code The language code
 */
org.ametys.skinfactory.SkinParametersManager.prototype.getLanguage = function (code)
{
	return this._languages[code];
}

/**
 * Get the current language code
 */
org.ametys.skinfactory.SkinParametersManager.prototype.getCurrentLanguage = function ()
{
	return this._currentLanguage;
}

/**
 * Update the current language
 * @param code The language code
 */
org.ametys.skinfactory.SkinParametersManager.prototype.updateCurrentLanguage = function (code)
{
	this._currentLanguage = code;
}

//------------------------------------------------------------------------------
org.ametys.skinfactory.SkinParametersManager.prototype._currentModel;

/**
 * Get the available languages.
 */
org.ametys.skinfactory.SkinParametersManager.prototype.getCurrentModel = function ()
{
	return this._currentModel;
}

/**
 * Update the current model
 * @param id the current model id
 */
org.ametys.skinfactory.SkinParametersManager.prototype.updateCurrentModel = function (id)
{
	this._currentModel = id;
}

//------------------------------------------------------------------------------
org.ametys.skinfactory.SkinParametersManager.prototype._cssStyleItems;
org.ametys.skinfactory.SkinParametersManager.prototype.getCssStyleItems = function (style)
{
	if (this._cssStyleItems == null)
	{
		this._cssStyleItems = {};
		
		var serverMessage = new org.ametys.servercomm.ServerMessage("skinfactory", 'skin/css-styles-items', {'siteName': context.parameters.siteName}, org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS, null, this, null, "text");
		var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
		
		// TODO Handle errors
		
		var result = Ext.util.JSON.decode(response[org.ametys.servercomm.ServerComm.xmlTextContent]);
		for (i in result)
		{
			this._cssStyleItems[i] = result[i];
		}
	}
	
	return this._cssStyleItems[style] != null ? this._cssStyleItems[style] : [];
}

